/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2011 ForTISS GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.base.ui.editpart;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.draw2d.AbsoluteBendpoint;
import org.eclipse.draw2d.BendpointConnectionRouter;
import org.eclipse.draw2d.Graphics;
import org.eclipse.draw2d.Label;
import org.eclipse.draw2d.MidpointLocator;
import org.eclipse.draw2d.PolygonDecoration;
import org.eclipse.draw2d.PolylineConnection;
import org.eclipse.emf.common.notify.Adapter;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.ecore.util.EContentAdapter;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.editpolicies.ConnectionEndpointEditPolicy;
import org.fortiss.tooling.base.model.base.ConnectionSegmentBase;
import org.fortiss.tooling.base.model.layout.LayoutPackage;
import org.fortiss.tooling.base.model.layout.Point;
import org.fortiss.tooling.base.model.layout.Points;
import org.fortiss.tooling.base.ui.editpart.policy.RemoveEditPolicy;
import org.fortiss.tooling.base.ui.layout.constants.DefaultLayoutConstants;
import org.fortiss.tooling.base.ui.layout.util.LayoutDataUtils;

/**
 * The base class for all layouted connection edit parts. This makes sure the
 * edit part is updated if the layout data changes in some way.
 * 
 * @param <T>
 *            the type of the model element wrapped.
 * 
 * @author hoelzl
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating YELLOW Hash: 4528B935FD404A834CBD215E46A1BFC1
 */
public abstract class LayoutedConnectionEditPartBase<T extends ConnectionSegmentBase>
		extends ConnectionEditPartBase<T> {

	/** The connection drawn. */
	protected PolylineConnection connection;

	/** The label drawn next to the connection. */
	protected final Label relationshipLabel = new Label(modelElement.getName());

	/** The adapter used for dealing with changes of the */
	// TODO (FH): check for collision with full refresh adapter of super class !
	protected final Adapter layoutChanged = new EContentAdapter() {

		@Override
		public void notifyChanged(Notification notification) {

			// skip everything that is not layout related
			if (notification.getNotifier() == modelElement) {
				if (notification.getFeature() != LayoutPackage.Literals.ILAYOUTED_MODEL_ELEMENT__LAYOUT_DATA) {
					return;
				}
			}

			super.notifyChanged(notification);
			refresh();
		}
	};

	/** Constructor. */
	protected LayoutedConnectionEditPartBase(T modelElement) {
		super(modelElement);
	}

	/** {@inheritDoc} */
	@Override
	protected void createEditPolicies() {
		installEditPolicy(EditPolicy.COMPONENT_ROLE, new RemoveEditPolicy());
		installEditPolicy(EditPolicy.CONNECTION_BENDPOINTS_ROLE,
				createBendPointPolicy());
		installEditPolicy(EditPolicy.CONNECTION_ENDPOINTS_ROLE,
				new ConnectionEndpointEditPolicy());
	}

	/** Updates the bend points, based on the model. */
	protected void refreshBendpoints() {
		Points pts = LayoutDataUtils.getConnectionPoints(modelElement);

		List<AbsoluteBendpoint> figureConstraint = new ArrayList<AbsoluteBendpoint>();
		int size = 0;
		if (pts != null) {
			size = pts.getPointsLength();
			for (int i = 0; i < size; i++) {
				Point pt = pts.getPoints(i);
				org.eclipse.draw2d.geometry.Point p = new org.eclipse.draw2d.geometry.Point(
						pt.getX(), pt.getY());

				figureConstraint.add(new AbsoluteBendpoint(p));
			}
		}

		// Update connection label
		if (connection.getChildren().contains(relationshipLabel)) {
			connection.remove(relationshipLabel);
		}
		relationshipLabel.setText(modelElement.getName());
		MidpointLocator mpl = new MidpointLocator(connection, size / 2) {
			@Override
			protected org.eclipse.draw2d.geometry.Point getReferencePoint() {
				org.eclipse.draw2d.geometry.Point p = super.getReferencePoint();
				p.y += DefaultLayoutConstants.DEFAULT_GRID_SIZE;
				return p;
			}
		};
		connection.add(relationshipLabel, mpl);

		getConnectionFigure().setRoutingConstraint(figureConstraint);
	}

	/** {@inheritDoc} */
	@Override
	protected PolylineConnection createFigure() {
		connection = new PolylineConnection();

		// arrow at target end point
		connection.setTargetDecoration(new PolygonDecoration());
		connection.setLineStyle(Graphics.LINE_SOLID);

		BendpointConnectionRouter connectionRouter = new BendpointConnectionRouter();
		connection.setConnectionRouter(connectionRouter);
		return connection;
	}

	/** Create connection specific bend point policy. */
	protected abstract EditPolicy createBendPointPolicy();

	/** {@inheritDoc} */
	@Override
	public void activate() {
		super.activate();
		modelElement.eAdapters().add(layoutChanged);
	}

	/** {@inheritDoc} */
	@Override
	public void deactivate() {
		modelElement.eAdapters().remove(layoutChanged);
		super.deactivate();
	}

	/** {@inheritDoc} */
	@Override
	protected void refreshVisuals() {
		refreshBendpoints();
	}
}
