/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2011 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.base.ui.layout.util;

import org.eclipse.draw2d.PositionConstants;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.fortiss.tooling.base.model.layout.ILayoutedModelElement;
import org.fortiss.tooling.base.ui.layout.constants.DefaultLayoutConstants;

/**
 * Utility methods for ellipse-shaped edit part layout.
 * 
 * @author hoelzl
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating RED Hash: 29FAC994522C13BC65F8538F36B6CFDA
 */
public final class EllipseLayoutUtils {
	/**
	 * Returns polar coordinates of the point on the ellipse border having the
	 * given angle.
	 */
	public static Polar calculateEllipseBorderPolar(double angle,
			Dimension ellipseSize, int inset) {
		double a = ellipseSize.width / 2 - inset;
		double b = ellipseSize.height / 2 - inset;
		double epsilon = Math.sqrt(a * a - b * b) / a;
		double r = b
				/ Math.sqrt(1 - epsilon * epsilon * Math.cos(angle)
						* Math.cos(angle));
		return new Polar(angle, r);
	}

	/**
	 * Returns Cartesian coordinates of the point on the ellipse border having
	 * the given angle.
	 */
	public static Point calculateEllipseBorderPoint(double angle,
			Dimension ellipseSize, int inset) {
		Polar p = calculateEllipseBorderPolar(angle, ellipseSize, inset);
		return p.toCartesian().translate(ellipseSize.getScaled(.5));
	}

	/** Layouts the given connector within the surrounding ellipse. */
	public static void layoutConnectorInEllipse(ILayoutedModelElement layouted,
			ILayoutedModelElement container, Point location) {
		Point position = location.getCopy();
		org.fortiss.tooling.base.model.layout.Point np = LayoutDataUtils
				.getNodePosition(container);
		position.translate(-np.getX(), -np.getY());

		org.fortiss.tooling.base.model.layout.Dimension nd = LayoutDataUtils
				.getNodeSize(container);
		Dimension d = new Dimension(nd.getWidth(), nd.getHeight());
		position.translate(d.scale(-.5));

		// store the value in the model
		LayoutDataUtils.setConnectorAngle(layouted, new Polar(position).angle);
	}

	/**
	 * Ensure that the minimum size and aspect ratio of a oval shape are
	 * redeemed.
	 */
	public static Dimension clampEllipseShapeSize(Dimension oldSize,
			int direction) {
		return clampEllipseShapeSize(oldSize, direction,
				DefaultLayoutConstants.DEFAULT_SHAPE_MINIMUM_WIDTH,
				DefaultLayoutConstants.DEFAULT_SHAPE_MINIMUM_HEIGHT,
				DefaultLayoutConstants.DEFAULT_SHAPE_ASPECT_RATIO);
	}

	/**
	 * Ensure that the minimum size and aspect ratio of a oval shape are
	 * redeemed.
	 */
	public static Dimension clampEllipseShapeSize(Dimension oldSize,
			int direction, int minimumWidth, int minimumHeight,
			double aspectRatio) {
		Dimension newSize;

		// check for minimum size
		newSize = new Dimension(oldSize.width < minimumWidth ? minimumWidth
				: oldSize.width, oldSize.height < minimumHeight ? minimumHeight
				: oldSize.height);

		// ensure that the aspect ratio is redeemed
		switch (direction) {
		case PositionConstants.SOUTH_EAST:
			int aspectHeight = (int) (newSize.width / aspectRatio);
			int aspectWidth = (int) (newSize.height * aspectRatio);
			newSize.setSize(new Dimension(
					newSize.width > aspectWidth ? aspectWidth : newSize.width,
					newSize.height > aspectHeight ? aspectHeight
							: newSize.height));
			break;
		case PositionConstants.SOUTH:
			newSize.setSize(new Dimension((int) (newSize.height * aspectRatio),
					newSize.height));
			break;
		case PositionConstants.EAST:
			newSize.setSize(new Dimension(newSize.width,
					(int) (newSize.width / aspectRatio)));
			break;
		}

		return newSize;
	}

	/**
	 * Class for storing polar coordinates.
	 * 
	 * @author wandinger
	 * @author hummel
	 * @author $Author$
	 * @version $Rev$
	 * @levd.rating YELLOW Rev: 519
	 */
	public static class Polar {

		/** The angle */
		public double angle;

		/** The distance */
		public double distance;

		/** The constructor */
		public Polar(double angle, double distance) {
			this.angle = angle;
			this.distance = distance;
		}

		/** Create new polar values from cartesian ones. */
		public Polar(Point pos) {
			angle = Math.atan2(pos.y, pos.x);
			distance = Math.sqrt(pos.x * pos.x + pos.y * pos.y);
		}

		/** Returns a cartesian representation. */
		public Point toCartesian() {
			return new Point(distance * Math.cos(angle), distance
					* Math.sin(angle));
		}
	}
}
