/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2011 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.extension;

import org.eclipse.emf.ecore.EObject;
import org.fortiss.tooling.kernel.extension.data.IConnectionCompositionContext;
import org.fortiss.tooling.kernel.service.IConnectionCompositorService;
import org.fortiss.tooling.kernel.service.base.IEObjectAware2;

/**
 * Interface for connection compositor extensions, i.e. classes which connect
 * and disconnect model elements. This may either be a direct connection or by
 * using an additional connection element.
 * <P>
 * Connection compositor extensions are handled by
 * {@link IConnectionCompositorService}.
 * <P>
 * A typical example is the creation and removal of an edge in a graph editor.
 * The connection compositor links two nodes of the graph and aggregates the
 * edge with the graph as its parent element.
 * 
 * @param <P>
 *            the meta-model class of the parent elements handled by this
 *            compositor.
 * @param <S>
 *            the meta-model class of source model elements handled by this
 *            compositor.
 * @param <T>
 *            the meta-model class of target model elements handled by this
 *            compositor.
 * 
 * @author hoelzl
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating GREEN Hash: FB4A39FA785FA53441998CE212CA9017
 */
public interface IConnectionCompositor<P extends EObject, S extends EObject, T extends EObject>
		extends IEObjectAware2<S, T> {
	/**
	 * Returns whether a connection between the source and the target is
	 * possible. Additional information can be included in the context, which
	 * also may be null. The connection parameter holds an existing connection
	 * in case of a reconnect.
	 */
	boolean canConnect(S source, T target, EObject connection,
			IConnectionCompositionContext context);

	/**
	 * Actually connects the given source and target. This will only be called
	 * if
	 * {@link #canConnect(EObject, EObject, EObject, IConnectionCompositionContext)}
	 * returned true. This method returns a boolean, since user-interaction
	 * during the connect might cancel the connect.
	 * 
	 * @param source
	 *            the source of the connection.
	 * @param target
	 *            the target of the connection.
	 * @param connection
	 *            the (existing) connection object. This may be null to indicate
	 *            that no connection object exists yet, or none is needed
	 *            (direct connection). If this is not null, this should be
	 *            reused for the connection (reconnect).
	 * @param context
	 *            context information for the connection. May be null.
	 */
	boolean connect(S source, T target, EObject connection,
			IConnectionCompositionContext context);

	/**
	 * Returns the parent element of the connection to be used during connecting
	 * the given elements.
	 */
	P getConnectionParent(S source, T target, EObject connection,
			IConnectionCompositionContext context);

	/**
	 * Returns whether a connection between the source and the target can be
	 * removed.
	 */
	boolean canDisconnect(EObject connection);

	/**
	 * Actually disconnects the given source and destination. This will only be
	 * called if {@link #canDisconnect(EObject)} returned true.
	 */
	boolean disconnect(EObject connection);
}
