/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2011 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.utils;

import java.util.List;

import org.eclipse.emf.ecore.EObject;
import org.fortiss.tooling.kernel.extension.data.ITransformationContext;
import org.fortiss.tooling.kernel.extension.data.TransformationProviderChain;
import org.fortiss.tooling.kernel.extension.exception.ChainTransformationFailedException;
import org.fortiss.tooling.kernel.service.ITransformationService;

/**
 * Utility methods for the {@link ITransformationService}.
 * 
 * @author hoelzl
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating RED Hash: 04773ED674972879DC273106533BBD9B
 */
public final class TransformationUtils {
	/**
	 * Searches a transformation chain for the given model element and the
	 * target class and returns the transformation result.
	 */
	@SuppressWarnings("unchecked")
	public static <T extends Object> T createTransformedObjectFor(
			final EObject modelElement, final Class<T> targetClass,
			ITransformationContext context)
			throws ChainTransformationFailedException {
		List<TransformationProviderChain> chainList = ITransformationService.INSTANCE
				.getTransformationProviderChain(modelElement.getClass(),
						targetClass, context);
		ChainTransformationFailedException e = null;
		if (!chainList.isEmpty()) {
			Object executableObject = null;
			for (TransformationProviderChain chain : chainList) {
				try {
					executableObject = chain.transform(modelElement, context);
					e = null;
				} catch (ChainTransformationFailedException ctfe) {
					// fall through
					e = ctfe;
					executableObject = null;
				}
			}
			if (e == null && executableObject != null) {
				return (T) executableObject;
			}
		}
		if (e != null) {
			throw e;
		}
		throw new ChainTransformationFailedException(null, null, null, null) {
			/** {@inheritDoc} */
			@Override
			public String getMessage() {
				return "No transformation chain found from " + modelElement
						+ " to " + targetClass.getName();
			}
		};
	}

	/**
	 * Searches a transformation chain for the given model element and the
	 * target class and returns the transformation result. This method returns
	 * <code>null</code> if the transformation failed. The exception is
	 * suppressed.
	 */
	public static <T extends Object> T createTransformedObjectWithoutExceptionFor(
			final EObject modelElement, final Class<T> targetClass,
			ITransformationContext context) {
		try {
			T result = createTransformedObjectFor(modelElement, targetClass,
					context);
			return result;
		} catch (ChainTransformationFailedException ctfe) {
			return null;
		}
	}
}
